#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "peri.h"
#include "usbdrv.h"

#define RQ_SET_LED         0
#define RQ_SET_LED_VALUE   1
#define RQ_GET_SWITCH1     2
#define RQ_GET_UV_LEVEL    3
#define RQ_GET_REF_LEVEL   4

//int wait = 0;
int state = 0;
int manual =0;
int i=0;
/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
     //Output from the sensor 
    
    static uint8_t switch_state;  
    static uint16_t uvLevel;
    static uint16_t refLevel;
    

    if (rq->bRequest == RQ_SET_LED)
    {
        uint8_t led_val = rq->wValue.bytes[0];
        uint8_t led_no  = rq->wIndex.bytes[0];
        set_led(led_no, led_val);
        return 0;
    }
    

    else if (rq->bRequest == RQ_GET_SWITCH1)
     {
        switch_state = IS_SWITCH_PRESSED();

        //point usbMsgPtr to the data to be returned to host 
        usbMsgPtr = &switch_state;

        //return the number of bytes of data to be returned to host 
        return 1;
    }
    
    else if (rq->bRequest == RQ_SET_LED_VALUE)
    {
            uint8_t led_val = rq->wValue.bytes[0];
            set_led_value(led_val);
	    return 0;
         
    }
    else if (rq->bRequest == RQ_GET_UV_LEVEL)
    {
        
        uvLevel = averageAnalogRead(4);
        usbMsgPtr = (uchar*)&uvLevel;
	return sizeof(uvLevel);


    }
     else if (rq->bRequest == RQ_GET_REF_LEVEL)
    {
        
        refLevel = averageAnalogRead(5);
        usbMsgPtr = (uchar*)&refLevel;
	return sizeof(refLevel);
     }
    
     
    /* default for not implemented requests: return no data back to host */
    return 0;
}
float getUV_Intensity(float x,float in_min,float in_max,float out_min,float out_max){
	return (x - in_min) * (out_max - out_min) / (in_max - in_min) + out_min;}

/* ------------------------------------------------------------------------- */
int main(void)
{   
    int clockwise =0;
    int counter_clockwise=1;
    int curtain_state=0;
    int wait = 0;
    int count_sw =0;
    init_peripheral();
    usbInit();
    
    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();
    /* main event loop */
     

    for(;;)
    {   
        usbPoll();
        float v_out = 3.3/averageAnalogRead(5)*averageAnalogRead(4);
        float uvIntensity = getUV_Intensity(v_out,0.99, 2.8, 0.0,15.0);
        int i=0;
      
       if(state==0)
          {
            while(IS_SWITCH_PRESSED())
               state=1;
          }
        else if(state==1) // RED ON
          {
            set_led_value(1);
            while(IS_SWITCH_PRESSED())
              { state=2; 
               }
         }
        else if(state==2) 
          {     
              while(IS_SWITCH_PRESSED())
               { set_led_value(4); //GREEN ON
                 state=3;// Auto Mode
               }                
           }
        else if(state==3)
           {
                while(IS_SWITCH_PRESSED())
               { set_led_value(2);
                 state=4;
               }
                  // Manual Mode
                
           }
        else if(state==4)
           {   
                if(count_sw >=0 && state==4)
                    {count_sw++;
                     if(count_sw==1000)
                        { state=5;
                          count_sw++;
                        }
                        
                    }
                
               
                while(IS_SWITCH_PRESSED() && state==4)
                   { state=6; 
                     count_sw =0;
                    }
                
           }
        else if(state==6)
           {
               while(IS_SWITCH_PRESSED())
               { set_led_value(0);
                 state=0;
               }      
           }
        
        
        
      /*.........................................................*/
       
       if(state==3)
        { if(uvIntensity>=9.00)
          {  set_led(GREEN,OFF);
             set_led(YELLOW,OFF); 
             wait++;
            
             if(wait==1000)  
             {	wait++;
                set_led(RED,ON);
               
                if(curtain_state==0)
                  {  
                     servo_loop(counter_clockwise);
                     curtain_state=1;
                     
                    
                  }
                 
                 if(curtain_state==1)
                  {  
                     servo_loop(counter_clockwise);
                     curtain_state=2;
                     
                     wait=0;
                     
                  }
                 if(curtain_state==2)
                     { curtain_state=2;
                     
                       wait=0;
                     
                     } 
             }
           }
       /*.........................................................*/
         if(uvIntensity>=3.00 && uvIntensity<9.00)
           { 
             set_led(GREEN,OFF);
             set_led(RED,OFF); 
             wait++;
             
             if(wait==1000)  
             {	wait++;
                set_led(YELLOW,ON);
                
                if(curtain_state==0)
                  {  
                     servo_loop(counter_clockwise);
                     curtain_state=1;
                     
                     
                  }
                  if(curtain_state==2)
                     { 
                       servo_loop(clockwise);
                       curtain_state=1;
                       
                       wait=0;
                    
                     } 
                 
                 if(curtain_state==1)
                  {  
                     curtain_state=1;
                     wait=0;

                  }
                
              }
           }
        /*.........................................................*/
       
       if(uvIntensity <3.00)
           { 
             set_led(YELLOW,OFF);
             set_led(RED,OFF); 
             wait++;
             
             if(wait==1000)  
             {	wait++;
                set_led(GREEN,ON);
                
                  
                  if(curtain_state==2)
                  {    
                       servo_loop(clockwise);
                       curtain_state=1;
                       
                        
                   }
                  
                 if(curtain_state==1)
                  {  
                     servo_loop(clockwise);
                     curtain_state=0;
                     wait=0;
                     
                  }
                  if(curtain_state==0)
                  { 
                     curtain_state=0;
                     wait=0;
                    
                  }
                
               }
            }
            
     /*.........................................................*/
         }
     
     if(state==5)
       { 
          if(curtain_state==0)
             { 
              while(IS_SWITCH_PRESSED())
                { servo_loop(counter_clockwise);
                  curtain_state=1;
                }
             }
            else if(curtain_state==1)
            {
             while(IS_SWITCH_PRESSED())
                { servo_loop(counter_clockwise);
                  curtain_state=2;
                }
             }
            else if(curtain_state==2)
            {
             while(IS_SWITCH_PRESSED())
                { servo_loop(counter_clockwise);
                  curtain_state=3;
                }
            }
            
            else if(curtain_state==3)
            {
             while(IS_SWITCH_PRESSED())
                { servo_loop(clockwise);
                  curtain_state=4;
                }
            }
            else if(curtain_state==4)
            {
             while(IS_SWITCH_PRESSED())
                { servo_loop(clockwise);
                  curtain_state=5;
                }
            }
            else if(curtain_state==5)
            {
             while(IS_SWITCH_PRESSED())
                { servo_loop(clockwise);
                  curtain_state=6;
                }
            }
           
        }
        if(curtain_state==6)
              { while(IS_SWITCH_PRESSED()) 
                { state=1;
                  curtain_state=0;        }
              }
        
        
     
   } 

     
 
    return 0;

    
}

/* ------------------------------------------------------------------------- */

